local L = LibStub("AceLocale-3.0"):GetLocale("NeedjaTracker", true)

NeedjaTracker = LibStub("AceAddon-3.0"):NewAddon("NeedjaTracker", "AceConsole-3.0", "AceEvent-3.0")

function NeedjaTracker:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("NeedjaTrackerDB")

    self:RegisterChatCommand("needjatracker", "ProcessChatCommand")
    self:RegisterChatCommand("nt", "ProcessChatCommand")

    self:RegisterEvent("CHAT_MSG_LOOT", "ProcessChatMessage")

    ChatFrame_AddMessageEventFilter("CHAT_MSG_LOOT", function(this, ...) return self:FilterChatMessage(...) end)

    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "ProcessPartyEvent")
    self:RegisterEvent("RAID_ROSTER_UPDATE", "ProcessPartyEvent")
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "ProcessInstanceEvent")

    self.watcher = NTWatch:New(self)
    self.gui = NTGui:New(self)
    self.reporter = NTReports:New(self)
end

function NeedjaTracker:ProcessChatCommand(msg)
    local args, opts = { self:GetArgs(msg, 100) }, {}
    local cmd = args[1]

    table.remove(args)
    table.remove(args, 1)

    local n, arg
    for n, arg in pairs(args) do opts[arg] = true end

    local channel
    if opts.party then
        channel = "PARTY"
    elseif opts.raid then
        channel = "RAID"
    elseif opts.guild then
        channel = "GUILD"
    end

    if cmd == "stat" or cmd == "statistics" then
        self.reporter:ShowRollStatistics(channel)
        if opts.clear then
            self.watcher:ClearRollStatistics()
        end
    elseif cmd == "hist" or cmd == "history" then
        self.reporter:ShowRollHistory(channel)
        if opts.clear then
            self.watcher:ClearRollStatistics()
        end
    elseif cmd == "clear" then
        self.watcher:ClearRollStatistics()
    else
        local i
        for i = 1, L.msg_help_total do
            PrintToChat(L["msg_help_" .. i])
        end
    end
end

function NeedjaTracker:ProcessChatMessage(event, msg)
    local handler, args = ParseMsgMatchTable(LOOT_ROLL_MSG_MATCH_TABLE, msg)

    if handler == "LOOT_ROLL_CHOSE" then
        self.watcher:EventChose(args.item, args.player, args.choice)

    elseif handler == "LOOT_ROLL_ROLLED" then
        self.watcher:EventRolled(args.item, args.player, args.choice, args.roll)

    elseif handler == "LOOT_ROLL_WON" then
        self.watcher:EventWon(args.item, args.player)

    elseif handler == "LOOT_ROLL_WON_NO_SPAM" then
        self.watcher:EventWinNoSpam(args.item, args.player, args.choice, args.roll)

    elseif handler == "LOOT_ROLL_ALL_PASSED" then
        self.watcher:EventAllPassed(args.item)
    end
end

function NeedjaTracker:FilterChatMessage(event, msg)
    return ParseMsgMatchTable(LOOT_ROLL_MSG_MATCH_TABLE, msg) ~= nil
end

function NeedjaTracker:ProcessPartyEvent()
    local db = self.db.char
    if IsInRaid() then
        if db.PartyStatus ~= 2 then
            if not db.PartyStatus then
                self.watcher:EventJoinGroup()
            end
            db.PartyStatus = 2
        end
    elseif IsInParty() then
        if db.PartyStatus ~= 1 then
            if not db.PartyStatus then
                self.watcher:EventJoinGroup()
            end
            db.PartyStatus = 1
        end
    else
        if db.PartyStatus then
            self.watcher:EventLeaveGroup()
            db.PartyStatus = nil
        end
    end
end

function NeedjaTracker:ProcessInstanceEvent()
    local db = self.db.char
    local instance = IsInInstance()
    if instance then
        if db.InstanceStatus ~= instance then
            self.watcher:EventEnterInstance()
            db.InstanceStatus = instance
        end
    end
end
